/*
    Koma Jump
        あかつきみさき

    このスクリプトについて
        タイムシートでのコマ表記からコンポジションの該当時間に移動するスクリプト.

    使用方法
        1.ファイル→スクリプト→スクリプトファイルの実行から実行.
        2.ScriptUIフォルダに入れてから,ウィンドウより実行.

    バージョン情報
        2024/03/06 Ver 1.0.0 Release
*/
/// <reference path="D:\lib\aftereffects.d.ts\ae.d.ts"/>

(function () {
    var data = {
        scriptName: "Koma Jump",
        scriptVersion: 20240306
    };

    Settings.prototype.initSetting = function (sectionName, keyName, defauleValue) {
        if (!app.settings.haveSetting(sectionName, keyName)) {
            app.settings.saveSetting(sectionName, keyName, defauleValue);
        }

        return app.settings.getSetting(sectionName, keyName);
    }

    var SHEET_PER_KOMA = app.settings.initSetting(data.scriptName, "SHEET_PER_KOMA", 144);

    var isCompActive = function (comp) {
        if (!(comp && comp instanceof CompItem)) {
            return false;
        } else {
            return true;
        }
    };

    var buildUI = function (thisObj) {
        var stjWin;

        if (thisObj instanceof Panel) {
            stjWin = thisObj;
        } else {
            stjWin = new Window("dialog", data.scriptName + " - " + data.scriptVersion, undefined, {
                resizeable: true
            });

            var actComp = app.project.activeItem;
            if (!isCompActive(actComp)) {
                alert("コンポジションをアクティブにして実行してください.", data.scriptName + " - " + data.scriptVersion);
                return 0;
            }
        }

        stjWin.orientation = "column";
        stjWin.spacing = 10;
        stjWin.margins = [10, 10, 10, 10];
        stjWin.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];

        var komaGrp = stjWin.add("group");
        komaGrp.orientation = "row";
        komaGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];

        var komaSt = komaGrp.add("statictext", undefined, "ページ+コマ :");
        komaSt.minimumSize.width = komaSt.maximumSize.width = 90;
        komaSt.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.FILL];

        var komaEt = komaGrp.add("edittext", undefined, "");
        komaEt.minimumSize.width = 100;
        komaEt.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        komaEt.active = true;
        komaEt.helpTip = "移動先の時間を指定します.\n例：1シート目の12コマ目に移動する場合は「12」か「1+12」を入力.";

        var shiftTimeGrp = stjWin.add("group");
        shiftTimeGrp.orientation = "row";
        shiftTimeGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];

        var shiftTimeSt = shiftTimeGrp.add("statictext", undefined, "シートの頭出し :");
        shiftTimeSt.minimumSize.width = shiftTimeSt.maximumSize.width = 90;
        shiftTimeSt.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.FILL];

        var shiftTimeEt = shiftTimeGrp.add("edittext", undefined, "0");
        shiftTimeEt.minimumSize.width = 100;
        shiftTimeEt.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        shiftTimeEt.helpTip = "シートの開始時間がずれる場合に指定します.\n例：先頭に12コマ追加され,元の1コマ目が13コマ目になる場合は「-12」を入力\n　　開始が12コマ削られ,13コマ目からになる場合は「12」を入力";

        var ssGrp = stjWin.add("group");
        ssGrp.orientation = "row";
        ssGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];

        // 1シート辺りのコマ数
        var sheetPerKomaSt = ssGrp.add("statictext", undefined, "1シートのコマ数 :");
        sheetPerKomaSt.minimumSize.width = sheetPerKomaSt.maximumSize.width = 90;
        sheetPerKomaSt.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.FILL];

        var sheetPerKomaEt = ssGrp.add("edittext", undefined, SHEET_PER_KOMA);
        sheetPerKomaEt.minimumSize.width = 100;
        sheetPerKomaEt.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        sheetPerKomaEt.helpTip = "1シート辺りの総コマ数を指定します.\n例：1秒辺り24コマの6秒シートの場合は144を入力.";
        sheetPerKomaEt.onChange = function () {
            app.settings.saveSetting(data.scriptName, "SHEET_PER_KOMA", this.text);
        }

        // ボタン
        var btnGrp = stjWin.add("group");
        btnGrp.orientation = "column";
        btnGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];

        var okBtn = btnGrp.add("button", undefined, "OK");
        okBtn.alignment = [ScriptUI.Alignment.CENTER, ScriptUI.Alignment.FILL];

        komaEt.onEnterKey = okBtn.onClick = function () {
            var actComp = app.project.activeItem;
            if (!isCompActive(actComp)) {
                alert("コンポジションをアクティブにして実行してください.", data.scriptName + " - " + data.scriptVersion);
                return 0;
            }

            var time = komaEt.text;
            var sheetPerKoma = sheetPerKomaEt.text;
            var shiftTime = shiftTimeEt.text;
            if (time == "") {
                stjWin.close();
                return 0;
            }

            if (sheetPerKoma == "") {
                stjWin.close();
                return 0;
            }

            if (shiftTime == "") {
                stjWin.close();
                return 0;
            }

            var sheet = [];

            if (isNaN(time)) {
                if (time == "" || time == null) {
                    alert("不正な値です.\n半角数字でコマ数のみか,「ページ+コマ」の形式で入力してください.", data.scriptName + " - " + data.scriptVersion, true);
                    komaEt.active = true;
                    return 0;
                }

                if (time.indexOf("+") == -1) {
                    alert("不正な値です.\n半角数字でコマ数のみか,「ページ+コマ」の形式で入力してください.", data.scriptName + " - " + data.scriptVersion, true);
                    komaEt.active = true;
                    return 0;
                }

                sheet = time.split("+");

                if (isNaN(sheet[0]) || isNaN(sheet[1]) || sheet[0] == "" || sheet[1] == "") {
                    alert("不正な値です.\n半角数字でコマ数のみか,「ページ+コマ」の形式で入力してください.", data.scriptName + " - " + data.scriptVersion, true);
                    komaEt.active = true;
                    return 0;
                }

                if (sheet[0] == 0) {
                    sheet[0] = 1;
                }
            } else {
                sheet = [1, time];
            }

            try {
                actComp.time = (Number(sheet[0] - 1) * Number(sheetPerKoma) + Number(sheet[1] - 1) + Number(shiftTime)) * actComp.frameDuration;
            } catch (err) {
                alert(err, data.scriptName + " - " + data.scriptVersion, true);
            }

            stjWin.close();
            return 0;
        }

        stjWin.layout.layout();
        stjWin.onResizing = stjWin.onResize = function () {
            this.layout.resize();
        }

        if (!(thisObj instanceof Panel)) {
            stjWin.center();
            stjWin.show();
        }

        return 0;
    }

    buildUI(this);
    return 0;
}).call(this);
